/*  XMMS2 - X Music Multiplexer System
 *  Copyright (C) 2003-2023 XMMS2 Team
 *
 *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
 */

#ifndef MODPLUG_COMPAT_H
#define MODPLUG_COMPAT_H

#ifdef HAVE_LIBMODPLUG_MODPLUG_H
# include <libmodplug/modplug.h>
#else
# include <modplug.h>
#endif

#endif /* MODPLUG_COMPAT_H */
